import React, { useState } from 'react'

import { documentToReactComponents } from '@contentful/rich-text-react-renderer'
import renderRichText from './renderRichText'
import { FormProvider, useForm } from 'react-hook-form'
import axios from 'axios'

import {
  Banner,
  CallToAction,
  Callout,
  Card,
  CardWithExtended,
  DActions,
  DFormSchema,
  Embed,
  Feature,
  Hero,
  Statistic,
  HomepageHero,
  Section,
  ExtendedSectionLeft,
  ExtendedSectionRight,
  SlantedSectionTop,
  Image,
  SmallIconRight,
  LatestNews,
} from 'law-centres-design-system'

import {
  Box,
  Button,
  Container,
  Divider,
  Heading,
  Link,
  SimpleGrid,
  Text,
  Stack,
  Flex,
  FormControl,
  FormLabel,
  Input,
} from '@chakra-ui/react'

const ContentTypes = {
  webform: (content, context, id) => {
    const methods = useForm()
    const [confirmation, setConfirmation] = useState()

    const onSubmit = (values) => {
      let data = Object.assign(values, { formId: id })

      axios.post('/api/forms/', data).then((res) => {
        if (res.data?.form?.confirmationMessage) {
          setConfirmation(res.data.form.confirmationMessage)
        } else {
          setConfirmation(
            'Sorry, something went wrong, please try again or contact support@lawcentres.org.uk'
          )
        }
      })
    }

    return (
      <Box maxW="xl" px="0" mb="5">
        {methods.formState.isSubmitSuccessful && (
          <>
            <Box textAlign="left">
              {confirmation && (
                <>
                  <Box mb="5">
                    <Heading color="gray.700" mb="5" size="md" fontWeight="800">
                      Form submitted
                    </Heading>

                    <ReactMarkdown // eslint-disable-line
                      components={{
                        a: (
                          { node, ...props } // eslint-disable-line
                        ) => (
                          <Link color="brand.Green" {...props}>
                            {props.children}
                          </Link>
                        ),
                        p: (
                          { node, ...props } // eslint-disable-line
                        ) => (
                          <Text color="gray.700" mb="3" {...props}>
                            {props.children}
                          </Text>
                        ),
                      }}
                    >
                      children={confirmation}
                    </ReactMarkdown>
                  </Box>
                  <Button
                    variant="simple"
                    // eslint-disable-next-line
                    onClick={(e) => {
                      methods.reset()
                      setConfirmation('')
                    }}
                  >
                    Back
                  </Button>
                </>
              )}
              {!confirmation && (
                <>
                  <Heading color="gray.700" mb="3" size="md" fontWeight="800">
                    Sending...
                  </Heading>
                </>
              )}
            </Box>
          </>
        )}
        {!methods.formState.isSubmitSuccessful && (
          <>
            <Heading color="gray.700" mb="7" size="md" fontWeight="800">
              {content.title}
            </Heading>
            <FormProvider {...methods}>
              <form onSubmit={methods.handleSubmit(onSubmit)}>
                <Box mb="5">
                  <DFormSchema useForm={methods} schema={content.fields} />
                </Box>
                <DActions />
              </form>
            </FormProvider>
          </>
        )}
      </Box>
    )
  },
  componentgrid: (content) => (
    <SimpleGrid
      position="relative"
      columns={[1, null, `${content.columns}`]}
      autoRows="max-content"
      spacing="10"
      _before={
        content.decoration
          ? {
              display: { base: 'none', xl: 'block' },
              content: '""',
              position: 'absolute',
              width: '20px',
              height: '20px',
              borderBottom: '20px solid #FCC24B',
              borderLeft: '20px solid #FCC24B',
              borderRight: 'none',
              borderTop: 'none',
              bottom: '-30px',
              left: '-30px;',
            }
          : {}
      }
      _after={
        content.decoration
          ? {
              display: { base: 'none', xl: 'block' },
              content: '""',
              position: 'absolute',
              width: '20px',
              height: '20px',
              borderBottom: 'none',
              borderLeft: 'none',
              borderRight: '20px solid #FCC24B',
              borderTop: '20px solid #FCC24B',
              top: '-30px;',
              right: '0px',
            }
          : {}
      }
    >
      <>
        {content.components.map((component, key) => (
          <Box key={key}>{renderComponent(component)}</Box>
        ))}
      </>
    </SimpleGrid>
  ),
  componentimage: (content) => (
    <Image
      src={'https:' + content.src.fields.file.url}
      alt={content.description}
    />
  ),
  componentdivider: () => <Divider borderColor="brand.darkBlue" opacity="1" />,
  containerhighlight: (content) => (
    <>
      {content.typeOfSection === 'Regular' && (
        <Section
          {...(content.anchor && { id: content.anchor })}
          pt="10rem"
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
        >
          <Container maxW="1250px" pb="150px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </Section>
      )}
      {content.typeOfSection === 'Angular Right' && (
        <ExtendedSectionRight
          {...(content.anchor && { id: content.anchor })}
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
          pt="5rem"
        >
          <Container maxW="1250px" pb="250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </ExtendedSectionRight>
      )}
      {content.typeOfSection === 'Angular Left' && (
        <ExtendedSectionLeft
          {...(content.anchor && { id: content.anchor })}
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
          pt="3rem"
        >
          <Container maxW="1250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </ExtendedSectionLeft>
      )}
      {content.typeOfSection === 'Slanted' && (
        <SlantedSectionTop
          {...(content.anchor && { id: content.anchor })}
          color={`brand.${content.textColour}`}
          backgroundColor={`brand.${content.backgroundColour}`}
        >
          <Container maxW="1250px" pb="250px">
            <Stack spacing="25px">
              {content.components.map((component, key) =>
                renderComponent(component, key)
              )}
            </Stack>
          </Container>
        </SlantedSectionTop>
      )}
    </>
  ),
  containercolumn: (content) => (
    <Box
      display="flex"
      justifyContent={content.centred ? 'center' : 'flex-start'}
    >
      <Box
        display="flex"
        position="relative"
        flexDirection="column"
        justifyContent={content.centred ? 'center' : 'flex-start'}
        spacing="10"
        width={content.width && { xl: `${content.width}%` }}
      >
        <>
          {content.components.map((component, key) => (
            <Box key={key}>{renderComponent(component)}</Box>
          ))}
        </>
      </Box>
    </Box>
  ),
  componentpageheader: (content) => (
    <Hero
      heading={content.heading}
      backgroundColour={`brand.${content.heroBackgroundColour}`}
      textColour={`brand.${content.textColour}`}
      pageTitle={content.pageTitle}
      subHeading={content.subHeading}
      image={content.heroImage?.fields?.file?.url}
      imageAlt={content.imageAlt}
      showCardDecoration={content.showCardDecoration}
    />
  ),
  componenthomepagehero: (content) => (
    <HomepageHero
      heading={content.heading}
      subHeading={content.subHeading}
      introduction={content.introduction}
      callToAction={content?.callToActionButton}
      callToActionLink={`/${content?.callToActionLink.fields.slug}`}
      heroImage={'https:' + content.heroImage.fields.file.url}
      heroImageAltText={content.heroImage.fields.file.description}
      headingBackground={content.headingBackground.fields.file.url}
    />
  ),
  componentrichtext: (content) => {
    let context = 'light'
    return (
      <Box pt="2" id={content.id}>
        {content &&
          documentToReactComponents(content.content, renderRichText(context))}
      </Box>
    )
  },
  componentfeature: (content) => (
    <Feature
      centred={content.centred}
      featureItems={content.components}
      textColor={`brand.${content.textColour}`}
    />
  ),
  componentlink: (content) => {
    const contentType = content?.link?.sys?.contentType.sys.id
    const linkSlug = content?.link?.fields.slug
    const externalLink = content?.externalLink

    let slug

    switch (true) {
      case !!externalLink:
        slug = externalLink
        break

      case contentType === 'pageSimple':
        slug = linkSlug === 'news' ? '/news' : `news/${linkSlug}`
        break

      default:
        slug = `/${linkSlug}`
        break
    }

    return (
      <Link
        target={content.newTab ? "_blank" : "_self"}
        href={slug}
        rel="noreferrer"
        textDecoration="underline"
        display="flex"
        alignItems="flex-start"
      >
        <SmallIconRight color={content.iconColour} width="12px" height="17px" />
        <Text fontWeight="bold" maxWidth="25ch" pl="1rem" mt="-6px">
          {content.text}
        </Text>
      </Link>
    )
  },
  componentbutton: (content) => {
    const contentType = content?.link?.sys.contentType.sys.id
    const linkSlug = content?.link?.fields.slug
    const externalLink = content?.externalLink

    let slug

    switch (true) {
      case !!externalLink:
        slug = externalLink
        break

      case contentType === 'pageSimple':
        slug = linkSlug === 'news' ? '/news' : `news/${linkSlug}`
        break

      default:
        slug = `/${linkSlug}`
        break
    }

    return (
      <Button
        variant={content?.variant}
        size={content?.size}
        colorScheme={content?.colorScheme}
        href={slug}
        as="a"
        target={content?.newTab ? '_blank' : '_self'}
        textDecoration="none"
      >
        {content?.buttonText}
      </Button>
    )
  },

  componentcallout: (content) => (
    <Callout
      backgroundColor={`brand.${content.backgroundColor}` || 'brand.darkBlue'}
      textColor={`brand.${content.textColor}` || 'brand.white.500'}
      type={content.type}
    >
      {content.components.map((component, key) => (
        <Box key={key}>{renderComponent(component)}</Box>
      ))}
    </Callout>
  ),
  componentcalltoaction: (content) => {
    const linkSlug = content?.link?.fields.slug
    const externalLink = content?.externalLink

    let slug = externalLink ? externalLink : linkSlug

    return (
      <Container maxW="1250px" py="10rem">
        <CallToAction
          heading={content.heading}
          description={content.description}
          buttonText={content.buttonText}
          link={slug}
        />
      </Container>
    )
  },
  componentstatistic: (content) => {
    return (
      <Box>
        <Statistic
          statisticOne={content.statisticOne}
          labelOne={content.labelOne}
          statisticTwo={content.statisticTwo}
          labelTwo={content.labelTwo}
          textColor={`brand.${content.textColor}`}
        />
      </Box>
    )
  },
  codeembed: (content) => (
    <Box pb="10" pt="5">
      <Embed code={content.embed} />
    </Box>
  ),
  componentbanner: (content) => (
    <Banner
      type={content.type}
      text={content.text}
      link={content.link}
      linkText={content.linkText}
    />
  ),

  componentcard: (content) => {
    const cardProps = {
      title: content.label,
      image: content.cardImage?.fields?.file?.url
        ? 'https:' + content.cardImage?.fields?.file?.url
        : '',
      alt: content?.cardImage?.fields?.title,
      link: content.link,
      newTab: content.newTab,
      body: content.cardBody,
      type: content.type,
      backgroundColor: `brand.${content.backgroundColor}` || 'brand.darkBlue',
      color: `brand.${content.textColor}` || 'brand.white.500',
      height: '100%',
    }

    return (
      <>
        {content.cardExtended ? (
          <CardWithExtended {...cardProps} />
        ) : (
          <Card {...cardProps} />
        )}
      </>
    )
  },

  componentsupportussection: (content) => {
    const [email, setEmail] = useState('')
    const [state, setState] = useState('idle')
    const [errorMsg, setErrorMsg] = useState(null)

    const subscribe = async (e) => {
      e.preventDefault()
      setState('Loading')

      try {
        const response = await axios.post('/api/actions/subscribe', { email })
        setState('Success')
        setEmail('')
        return response
      } catch (e) {
        console.log(e.response.data.error)
        setErrorMsg(e.response.data.error)
        setState('Error')
      }
    }
    const contentType = content?.callToActionLink?.sys?.contentType.sys.id
    const linkSlug = content?.callToActionLink?.fields.slug
    const externalLink = content?.externalLink

    let slug

    switch (true) {
      case !!externalLink:
        slug = externalLink
        break

      case contentType === 'pageSimple':
        slug = linkSlug === 'news' ? '/news' : `news/${linkSlug}`
        break

      default:
        slug = `/${linkSlug}`
        break
    }

    return (
      <Flex
        as="section"
        width="100%"
        flexWrap="wrap"
        direction={{ base: 'column', xl: 'row' }}
      >
        <Flex
          bg="brand.legacyTeal"
          paddingBottom="86px"
          width={{ base: '100%', xl: '55%' }}
          alignItems="center"
          justifyContent="center"
        >
          <Stack
            spacing="3.125rem"
            paddingInlineStart="1rem"
            py="4rem"
            pr="2rem"
          >
            <Text
              color="brand.white.500"
              fontFamily="bodyAlternative"
              fontSize={{ base: 'xl', xl: 'xl' }}
              lineHeight="50px"
              fontWeight="600"
              maxWidth="534px"
            >
              {content.callToActionText}
            </Text>
            <Button as="a" href={slug}>
              {content.callToActionButton}
            </Button>
          </Stack>
        </Flex>

        <Flex
          bg="brand.brightTeal"
          justifyContent="center"
          width={{ base: '100%', xl: '45%' }}
        >
          <Stack spacing="2.125rem" paddingInlineEnd="1rem" py="4rem" pl="2rem">
            <Text fontSize="2xs" maxWidth="345px">
              {content.subscribeToNewsletterText}
            </Text>

            <Stack width="100%" as="form" spacing="25px">
              <FormControl>
                <FormLabel>Your email</FormLabel>
                <Input
                  type="email"
                  name="email"
                  value={email}
                  onChange={(e) => setEmail(e.target.value)}
                />
              </FormControl>
              <Button
                as="a"
                href="https://www.google.com"
                size="short"
                disabled={state === 'Loading'}
                type="submit"
                variant="outline"
                colorScheme="brand.darkBlue"
                onClick={subscribe}
              >
                Sign up
              </Button>
              {state === 'Error' && <Text>{errorMsg}</Text>}
              {state === 'Success' && <Text>You&apos;ve been subscribed!</Text>}
            </Stack>
          </Stack>
        </Flex>
      </Flex>
    )
  },

  componentdownloadcard: (content) => {
    const cardProps = {
      title: content.label,
      image: content.cardImage?.fields?.file?.url
        ? 'https:' + content.cardImage?.fields?.file?.url
        : '',
      height: '100%',
      alt: content?.cardImage?.fields?.title,
      downloadFileURL: 'https:' + content?.downloadFile?.fields?.file?.url,
      downloadFileTitle: content?.downloadFile?.fields?.title,
      body: content.cardBody,
      type: content.type,
      backgroundColor: `brand.${content.backgroundColor}` || 'brand.darkBlue',
      color: `brand.${content.textColor}` || 'brand.white.500',
    }

    return (
      <>
        {content.cardExtended ? (
          <CardWithExtended {...cardProps} />
        ) : (
          <Card {...cardProps} />
        )}
      </>
    )
  },

  page: (content) => {
    return (
      <Card
        title={content.title}
        image={content.seo.fields.image?.fields?.file?.url}
        link={content.slug}
      >
        <Text>{content.seo.fields.description}</Text>
      </Card>
    )
  },
  patternlatestnews: (component, articles) => {
    return (
      <LatestNews
        articles={articles}
        backgroundColor="brand.brightPink"
        cardBackgroundColor="brand.brightTeal"
        headingColor="brand.white.500"
        cardTextColor="brand.darkBlue"
        backgroundImage={'https:' + component.backgroundImage.fields.file.url}
      />
    )
  },
}

function renderComponent(component, articles, context = 'light') {
  const type = component.__typename || component.sys.contentType.sys.id
  const content = component.fields || component
  let id = component.sys.id

  try {
    let rendered = ContentTypes[type.toLowerCase()](
      content,
      articles,
      context,
      id
    )
    return rendered
  } catch (err) {
    console.log(`Can not render ${type}, ${err}`)
  }
}

export { renderComponent, ContentTypes }
